/*
 * Qt5-UKUI's Library
 *
 * Copyright (C) 2023, KylinSoft Co., Ltd.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: Jing Tan <tanjing@kylinos.cn>
 *
 */

import QtQuick 2.6
import QtQuick.Templates 2.5 as T
import org.ukui.qqc2style.private 1.0 as StylePrivate


T.CheckBox {
    id: control

    implicitWidth: Math.max(implicitBackgroundWidth + leftInset + rightInset,
                            implicitContentWidth + leftPadding + rightPadding)
    implicitHeight: Math.max(implicitBackgroundHeight + topInset + bottomInset,
                             implicitContentHeight + topPadding + bottomPadding,
                             implicitIndicatorHeight + topPadding + bottomPadding)

    padding: 6
    spacing: checkbox.space
    hoverEnabled: true

    property var borderColor: control.checked ? (!control.enabled ? checkbox.checked_disableIndicatorBorderColor :
                                                                    control.pressed ? checkbox.checked_clickIndicatorBorderColor : control.hovered ?
                                                                    checkbox.checked_hoverIndicatorBorderColor : checkbox.checked_normalIndicatorBorderColor) :
                                                                   (!control.enabled ? checkbox.disableIndicatorBorderColor : control.pressed ?
                                                                    checkbox.clickIndicatorBorderColor : control.hovered ?
                                                                    checkbox.hoverIndicatorBorderColor : checkbox.normalIndicatorBorderColor)

    property var checkBoxColor: control.checked ? (!control.enabled ? checkbox.checked_disableIndicatorColor :
                                                                        control.pressed ? checkbox.checked_clickIndicatorColor : control.hovered ?
                                                                        checkbox.checked_hoverIndicatorColor : checkbox.checked_normalIndicatorColor) :
                                                                       (!control.enabled ? checkbox.disableIndicatorColor : control.pressed ?
                                                                        checkbox.clickIndicatorColor : control.hovered ?
                                                                        checkbox.hoverIndicatorColor : checkbox.normalIndicatorColor)
    ParseInterface{
        id: parseInterface
    }

    function getStartColor(dtcolor)
    {
        return parseInterface.pStartColor(dtcolor)
    }
    function getEndColor(dtcolor)
    {
        return parseInterface.pEndColor(dtcolor)
    }

    // keep in sync with CheckDelegate.qml (shared CheckIndicator.qml was removed for performance reasons)
    indicator: Rectangle {
        implicitWidth: checkbox.indicatorWidth
        implicitHeight: checkbox.indicatorWidth

        x: control.text ? (control.mirrored ? control.width - width - control.rightPadding : control.leftPadding) : control.leftPadding + (control.availableWidth - width) / 2
        y: control.topPadding + (control.availableHeight - height) / 2
        radius: checkbox.radius

        border.width: checkbox.borderWidth
        border.color: getStartColor(borderColor)
        color: gradientRec.visible ? "transparent" : getStartColor(checkBoxColor)

        Rectangle{
            id: gradientRec
            anchors.fill: parent
            radius: parent.radius
            border.width: parent.border.width
            border.color: parent.border.color
            gradient:  Gradient {
                id:gradient
                GradientStop { position: 0.0; color: getStartColor(checkBoxColor) }

                GradientStop { position: 1.0; color: getEndColor(checkBoxColor)}
            }
            visible: !parseInterface.isSolidPattern(checkBoxColor)
        }

        Image {
            x: (parent.width - width) / 2
            y: (parent.height - height) / 2
            width: 14
            height: 12
//            defaultColor: "#353637"
//            color: control.palette.text
            source: {
                var model = !control.enabled ? "disenable" : control.checked ? "highlight" : "normal"                
                var icon = "object-select-symbolic";
                if(!app.themeHasIcon(icon))
                    icon = "file:///usr/share/icons/ukui-icon-theme-default/scalable/actions/object-select-symbolic.svg"
                return "image://imageProvider/" + icon + "/" + model;

            }
            visible: control.checkState === Qt.Checked
        }

        Rectangle {
            x: (parent.width - width) / 2
            y: (parent.height - height) / 2
            width: 9
            height: 1
            color: {
                getStartColor(!control.enabled ? checkbox.checked_disableChildrenColor :
                                   control.pressed ? checkbox.checked_clickChildrenColor:
                                                     control.hovered ? checkbox.checked_hoverChildrenColor:
                                                                       checkbox.checked_normalChildrenColor)
            }
            visible: control.checkState === Qt.PartiallyChecked
        }
    }

    contentItem: Label {
        leftPadding: control.indicator && !control.mirrored ? control.indicator.width + control.spacing : 0
        rightPadding: control.indicator && control.mirrored ? control.indicator.width + control.spacing : 0

        text: control.text
        font: app.font
        color: getStartColor(control.enabled ? checkbox.normalTextColor : checkbox.disableTextColor)
    }

    StylePrivate.APPParameter{
        id: app
        onParametryChanged:{
            control.update();
        }
    }
    StylePrivate.UKUICheckBox{
        id: checkbox
    }
}
