/*
 * Qt5-UKUI's Library
 *
 * Copyright (C) 2023, KylinSoft Co., Ltd.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: Jing Tan <tanjing@kylinos.cn>
 *
 */

import QtQuick 2.15
import QtQuick.Window 2.15
import QtQuick.Controls 2.15
import QtGraphicalEffects 1.0
import QtQuick.Controls.impl 2.15
import QtQuick.Templates 2.15 as T
import org.ukui.qqc2style.private 1.0 as StylePrivate

T.ComboBox {
    id: control

    implicitWidth: Math.max(implicitBackgroundWidth + leftInset + rightInset,
                            implicitContentWidth + leftPadding + rightPadding)
    implicitHeight: Math.max(implicitBackgroundHeight + topInset + bottomInset,
                             implicitContentHeight + topPadding + bottomPadding,
                             implicitIndicatorHeight + topPadding + bottomPadding)
    padding: combobox.leftRightPadding
    leftPadding: padding + (!control.mirrored || !indicator || !indicator.visible ? 0 : indicator.width + spacing)
    rightPadding: padding + (control.mirrored || !indicator || !indicator.visible ? 0 : indicator.width + spacing)
    hoverEnabled: true

    ParseInterface{
        id: parseInterface
    }

    function getStartColor(dtcolor)
    {
        return parseInterface.pStartColor(dtcolor)
    }
    function getEndColor(dtcolor)
    {
        return parseInterface.pEndColor(dtcolor)
    }

    property var scrollbarWidth : 0
    delegate: MenuItem {
        id: menuitem
        implicitWidth: control.width - 2 * control.leftPadding - scrollbarWidth
        text: control.textRole ? (Array.isArray(control.model) ? modelData[control.textRole] : model[control.textRole]) : modelData
        icon.name: (control.iconNameRole && model[control.iconNameRole] !== undefined) ? model[control.iconNameRole] : null
        highlighted: control.highlightedIndex === index
        hoverEnabled: control.hoverEnabled
        autoExclusive: true
        checked: control.currentIndex === index
        Component.onCompleted: {
//            console.log("delegate menuitem......", control.width, control.leftPadding, control.rightPadding, scrollbarWidth)
            if(menuitem.hasOwnProperty("itemHasChecked")){
                itemHasChecked = true
            }
        }
    }

    contentItem: T.TextField {
        leftPadding: 0//!control.mirrored ? 12 : control.editable && activeFocus ? 3 : 1
        rightPadding:0// control.mirrored ? 12 : control.editable && activeFocus ? 3 : 1
        anchors.verticalCenter: control.verticalCenter

        text: control.editable ? control.editText : control.displayText

        enabled: control.editable
        autoScroll: control.editable
        readOnly: control.down
        inputMethodHints: control.inputMethodHints
        validator: control.validator
        selectByMouse: control.selectTextByMouse

        font: app.font
        color: getStartColor(!control.enabled ? combobox.disableTextColor : combobox.normalTextColor)
        selectionColor: control.palette.highlight
        selectedTextColor: control.palette.highlightedText
        verticalAlignment: Text.AlignVCenter
    }

    indicator: Image {
        id: indicatorImage
        x: control.mirrored ? control.padding : control.width - width - control.padding
        y: control.topPadding + (control.availableHeight - height) / 2
        z: 10
        width: app.iconWidth
        height: width
        source: {
            var icon = "ukui-down-symbolic";
            if(!pop.visible){
                if(!app.themeHasIcon(icon))
                    icon = "file:///usr/share/icons/ukui-icon-theme-default/scalable/actions/ukui-down-symbolic.svg"
            }
            else{
                icon = "ukui-up-symbolic"
                if(!app.themeHasIcon(icon))
                    icon = "file:///usr/share/icons/ukui-icon-theme-default/scalable/actions/ukui-up-symbolic.svg"
            }

            var model = !control.enabled ? "disenable" : "normal"

            return "image://imageProvider/" + icon + "/" + model;

        }

        MouseArea{
        anchors.fill: parent
        onClicked:
        {
//            popHandle.setPosion(parent.mapToGlobal(control.x, control.y + control.height).x, parent.mapToGlobal(control.x, control.y + control.height).y)

//            if(popHandle && !popHandle.isVisible()){
//                popHandle.open()
//            }
//            else if(popHandle && popHandle.isVisible())
//                popHandle.close()
            pop.visible = !pop.visible;
        }
        }
    }

    background: Rectangle {
        implicitWidth: combobox.normalWidth
        implicitHeight: combobox.normalHeight
        radius: combobox.radius

        color: gradientRec.visible ? "transparent" : getStartColor(combobox.normalBC)
        border.color: getStartColor(!control.enabled ? combobox.disableBorderColor :
                                     control.focus ? combobox.focusBorderColor : combobox.normalBorderColor)
        border.width: control.focus ? combobox.focusBorderWidth : combobox.borderWidth;


        Rectangle{
            id: gradientRec
            anchors.fill: parent
            radius: parent.radius
            border.width: parent.border.width
            border.color: parent.border.color
            gradient:  Gradient {
                id:gradient
                GradientStop { position: 0.0; color: getStartColor(combobox.normalBC) }

                GradientStop { position: 1.0; color: getEndColor(combobox.normalBC)}
            }
            visible: !parseInterface.isSolidPattern(combobox.normalBC)
        }
    }



    popup: Popup {
        id: pop
        y: control.height
        width: control.width
//        height: Math.min(contentItem.implicitHeight + topMargin + bottomMargin, control.Window.height - topMargin - bottomMargin)
        topMargin: 6
        bottomMargin: 6
        background: Item{}

        onVisibleChanged: {
            if(popHandle && parent)
            {
                popHandle.setPosion(mapToGlobal(x, y).x, mapToGlobal(x, y).y)
            }
            if(visible && popHandle && !popHandle.visible){
                popHandle.open()
            }
            else if(!visible && popHandle && popHandle.visible)
                popHandle.close()
        }
        onYChanged: {
            if(popHandle && parent)
            {
                popHandle.setPosion(mapToGlobal(x, y).x, mapToGlobal(x, y).y)
            }
        }
        onXChanged: {
            if(popHandle && parent)
            {
                popHandle.setPosion(mapToGlobal(x, y).x, mapToGlobal(x, y).y)
            }
        }

        StylePrivate.PopupHandle{
            id: popHandle
            onVisibleChanged: {
                //            popupwindow.visible = visible
                    if(pop.visible !== visible){
                        pop.visible = visible
                    }
            }
            backColor: ukuiPopup.backColor
            shadowColor: ukuiPopup.shadowColor

            delegate: Window {
                id: popupwindow
    //                width: windowLabel ? windowLabel.width : 0
    //                height: windowLabel ? windowLabel.height : 0
                //        blurControl: control
                flags: Qt.ToolTip


                Component.onCompleted: {
                    scrollbar.visible = popHandle.outScreen
                }

                Rectangle{
                    id: popRec
                    width: parent.width
                    height: parent.height
                    visible: true
                    radius: ukuiPopup.radius
                    color: getStartColor(ukuiPopup.backColor)
                    //border.color: StylePrivate.ParseColorInterface.startColor(ukuiPopup.backBorderColor)
                    opacity: app.menuTransparency
                    layer.enabled: true

                    ScrollBar {
                        id: scrollbar
                        active: visible
                        orientation: Qt.Vertical
                        visible: popHandle.outScreen
                        anchors.top: parent.top
                        anchors.topMargin: pop.topPadding
                        anchors.right: parent.right
                        height: listview.height
                        position: listview.visibleArea.yPosition
                        size: listview.visibleArea.heightRatio

                        onPositionChanged: {
                            if (active && listview.visibleArea.yPosition != position) {
                                vscrollbar.position = position;
                            }
                        }
                        onVisibleChanged:{
                            if(visible)
                            scrollbarWidth = scrollbar.width
                        }
                    }
                    layer.effect: DropShadow {
                        transparentBorder: true
                        radius: ukuiPopup.radius
                        samples: 16
                        horizontalOffset: 0
                        verticalOffset: 0
                        width: popRec.width + 8
                        height: popRec.height + 8
                        color: StylePrivate.ParseColorInterface.startColor(ukuiPopup.shadowColor)
                    }
                }


                Connections {
                    target: popHandle
                    function onVisibleChanged(visible) {
                        if(visible){
                            listview.visible = true
                            popupwindow.show()
                        }
                        else{
                            listview.visible = false
                            popupwindow.hide()
                        }
                    }
                    function onSizeChanged(width, height){
                        //                        pop.implicitWidth = width
                        if(listview.implicitHeight !== height)
                            listview.implicitHeight = height
                        if(popRec.width !== width)
                            popRec.width = width
                        if(popRec.height !== height)
                            popRec.height = height
                    }

                }

                ListView {
                    id: listview
                    clip: popHandle.outScreen
                    implicitHeight: contentHeight
                    implicitWidth: control.width  - 2 * control.leftPadding - scrollbarWidth
                    model: control.delegateModel
                    currentIndex: control.highlightedIndex
                    highlightMoveDuration: 0
                    interactive: popHandle.outScreen
                    anchors.top: popRec.top
                    anchors.topMargin: combobox.upDownMargin
                    anchors.left: popRec.left
                    anchors.leftMargin: combobox.upDownMargin
                    onVisibleChanged: {
                        if(visible){
                            listview.implicitHeight = listview.contentHeight
                            popHandle.setSize(control.width, listview.contentHeight + 2 * combobox.upDownMargin)
                        }
                    }

                    onWidthChanged: {
    //                    pop.height = listview.implicitHeight + 2 * combobox.upDownMargin

                        popHandle.setSize(control.width, listview.contentHeight + 2 * combobox.upDownMargin)
                    }
                    onHeightChanged: {
    //                    pop.height = listview.implicitHeight + 2 * combobox.upDownMargin

                        popHandle.setSize(control.width, listview.contentHeight + 2 * combobox.upDownMargin)
                    }
                    Component.onCompleted: {
    //                    pop.height = listview.implicitHeight + 2 * combobox.upDownMargin

                        popHandle.setSize(control.width, listview.contentHeight + 2 * combobox.upDownMargin)
                    }
                    onContentHeightChanged: {
                        listview.implicitHeight = listview.contentHeight
                        popHandle.setSize(control.width, listview.contentHeight + 2 * combobox.upDownMargin)
                    }

                    ScrollBar.vertical: ScrollBar {
                        id: vscrollbar
                        active: visible
                        visible: popHandle ? popHandle.outScreen : false
                        width: 0
                    }


                    Connections{
                        target: popHandle
                        function onOutScreenChanged()
                        {
                            listview.interactive = popHandle.outScreen ? true : false;
                            listview.clip = listview.interactive;
                            scrollbar.visible = popHandle.outScreen;
                        }
                    }
                }
            }
        }
    }

    StylePrivate.UKUIPopup{
        id: ukuiPopup
    }

    StylePrivate.UKUIComboBox{
        id: combobox
    }

    StylePrivate.APPParameter{
        id: app
    }

}
